#!/bin/sh

# mktorfw.sh - create an linux iptables firewall for tor client

#
# Copyright (c) 2007, Gregory Fleischer (gfleischer@gmail.com)
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# 3. Neither the name of the author nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

set -e -u

IPTABLES="/sbin/iptables -v"
EXT_IFACE="eth1"

$IPTABLES -P INPUT DROP
$IPTABLES -P OUTPUT DROP
$IPTABLES -P FORWARD DROP

for i in filter; do
    $IPTABLES -t $i -F
    $IPTABLES -t $i -X
done

# reject
$IPTABLES -N reject_and_log
$IPTABLES -A reject_and_log -j LOG --log-prefix "rejecting packet: "
$IPTABLES -A reject_and_log -p tcp -j REJECT --reject-with tcp-reset
$IPTABLES -A reject_and_log -j REJECT --reject-with icmp-port-unreachable

# drop
$IPTABLES -N drop_and_log
$IPTABLES -A drop_and_log -j LOG --log-prefix "dropping packet: "
$IPTABLES -A drop_and_log -j DROP

# local ports
$IPTABLES -N local_ports
$IPTABLES -A local_ports -p tcp -d localhost --dport 8118 -j ACCEPT
$IPTABLES -A local_ports -p tcp -d localhost --dport 9050 -j ACCEPT
$IPTABLES -A local_ports -p tcp -d localhost --dport 4443 -j ACCEPT
$IPTABLES -A local_ports -p tcp -d localhost --dport 5553 -j ACCEPT
$IPTABLES -A local_ports -p tcp -d localhost --dport 5533:5539 -j ACCEPT

#### begin check
$IPTABLES -N is_tor
script=`cat /var/lib/tor/cached-routers* | grep "^router " | \
    awk '(!$4~/0/) {print "$IPTABLES -A is_tor -p tcp -d "$3" --dport "$4" -j ACCEPT"} \
         (!$6~/0/) {print "$IPTABLES -A is_tor -p tcp -d "$3" --dport "$6" -j ACCEPT"}'`
eval "$script"
#### end check

# localhost 8118 and 9050
$IPTABLES -A INPUT -m state --state INVALID -j drop_and_log
$IPTABLES -A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
$IPTABLES -A INPUT -j local_ports 
$IPTABLES -A INPUT -p icmp -s localhost -j ACCEPT
$IPTABLES -A INPUT -j reject_and_log

$IPTABLES -A OUTPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
$IPTABLES -A OUTPUT -j local_ports 
$IPTABLES -A OUTPUT -p icmp -d localhost -j ACCEPT
$IPTABLES -A OUTPUT -j is_tor
$IPTABLES -A OUTPUT -j reject_and_log

# eof
